package db

import (
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectUserVirtualCoinFlowAggregations 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `UserVirtualCoinFlowAggregationFindByParams` 方法
func BatchSelectUserVirtualCoinFlowAggregations(Db *xorm.Engine, params map[string]interface{}) (*[]model.UserVirtualCoinFlowAggregation, error) {
	var UserVirtualCoinFlowAggregationData []model.UserVirtualCoinFlowAggregation
	if err := Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&UserVirtualCoinFlowAggregationData); err != nil {
		return nil, logx.Warn(err)
	}
	return &UserVirtualCoinFlowAggregationData, nil
}

// UserVirtualCoinFlowAggregationInsert 插入单条数据
func UserVirtualCoinFlowAggregationInsert(Db *xorm.Engine, UserVirtualCoinFlowAggregation *model.UserVirtualCoinFlowAggregation) (int64, error) {
	_, err := Db.InsertOne(UserVirtualCoinFlowAggregation)
	if err != nil {
		return 0, err
	}
	return UserVirtualCoinFlowAggregation.Id, nil
}

// BatchAddUserVirtualCoinFlowAggregations 批量新增数据
func BatchAddUserVirtualCoinFlowAggregations(Db *xorm.Engine, UserVirtualCoinFlowAggregationData []*model.UserVirtualCoinFlowAggregation) (int64, error) {
	affected, err := Db.Insert(UserVirtualCoinFlowAggregationData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetUserVirtualCoinFlowAggregationCount(Db *xorm.Engine) int {
	var UserVirtualCoinFlowAggregation model.UserVirtualCoinFlowAggregation
	session := Db.Where("")
	count, err := session.Count(&UserVirtualCoinFlowAggregation)
	if err != nil {
		return 0
	}
	return int(count)
}

// UserVirtualCoinFlowAggregationDelete 删除记录
func UserVirtualCoinFlowAggregationDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.UserVirtualCoinFlowAggregation{})
	} else {
		return Db.Where("id = ?", id).Delete(model.UserVirtualCoinFlowAggregation{})
	}
}

// UserVirtualCoinFlowAggregationUpdate 更新记录
func UserVirtualCoinFlowAggregationUpdate(Db *xorm.Engine, id interface{}, UserVirtualCoinFlowAggregation *model.UserVirtualCoinFlowAggregation, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = Db.Where("id=?", id).Cols(forceColums...).Update(UserVirtualCoinFlowAggregation)
	} else {
		affected, err = Db.Where("id=?", id).Update(UserVirtualCoinFlowAggregation)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// UserVirtualCoinFlowAggregationGetOneByParams 通过传入的参数查询数据(单条)
func UserVirtualCoinFlowAggregationGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.UserVirtualCoinFlowAggregation, error) {
	var m model.UserVirtualCoinFlowAggregation
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := Db.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return &m, nil
}

// UserVirtualCoinFlowAggregationFindByParams 通过传入的参数查询数据(多条)
func UserVirtualCoinFlowAggregationFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.UserVirtualCoinFlowAggregation, error) {
	var m []model.UserVirtualCoinFlowAggregation
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}
