package db

import (
	"applet/app/db/model"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"fmt"
	"reflect"
	"strings"
	"xorm.io/xorm"
)

// VirtualCoinGetOneByParams 通过参数查询数据（单条）
func VirtualCoinGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.VirtualCoin, error) {
	var m model.VirtualCoin
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}
func VirtualCoinGetOneByParamsSess(sess *xorm.Session, params map[string]interface{}) (*model.VirtualCoin, error) {
	var m model.VirtualCoin
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := sess.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}
func VirtualCoinGetOneById(Db *xorm.Engine, id int) (*model.VirtualCoin, error) {
	var m model.VirtualCoin
	if has, err := Db.Where("id=?", id).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// VirtualCoinFindByParams 通过传入的参数查询数据(多条)
func VirtualCoinFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.VirtualCoin, error) {
	var m []model.VirtualCoin
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}

// VirtualCoinListInUseAndNoBlockCoin 查询正在使用中的虚拟币（排除区块币）
func VirtualCoinListInUseAndNoBlockCoin(Db *xorm.Engine, masterId string) ([]*model.VirtualCoin, error) {
	var m []*model.VirtualCoin
	cacheKey := fmt.Sprintf(md.VirtualCoinCfgCacheKey, masterId)

	err := cache.GetJson(cacheKey, &m)
	if err != nil || len(m) == 0 {
		// is_block=0 非作为区块币的才能用与返利
		err := Db.Where("is_use=1 AND is_block=0").Find(&m)
		if err != nil {
			return nil, err
		}
		cache.SetJson(cacheKey, m, md.CfgCacheTime)
	}

	return m, nil
}

// VirtualCoinListInUse 查询正在使用中的虚拟币
func VirtualCoinListInUse(Db *xorm.Engine, masterId, isFreeze string) ([]*model.VirtualCoin, error) {
	var m []*model.VirtualCoin
	cacheKey := fmt.Sprintf(md.VirtualCoinCfgCacheKey, masterId)

	err := cache.GetJson(cacheKey, &m)
	if err != nil || len(m) == 0 {
		err := Db.Where("is_use=1").Asc("id").Find(&m)
		if err != nil {
			return nil, err
		}
		cache.SetJson(cacheKey, m, md.CfgCacheTime)
	}
	if masterId == "88164961" {
		for k, v := range m {
			if v.Id == 2 {
				if strings.Contains(v.CanExchange, "{\"id\":") {
					m[k].CanExchange = strings.ReplaceAll(v.CanExchange, "{\"id\":", "{\"id\":\"4\",\"name\":\"购物积分\",\"fee\":\"0\",\"coin_id\":\"4\"},{\"id\":")
				}
				if v.CanExchange == "[]" {
					m[k].CanExchange = "[{\"id\":\"4\",\"name\":\"购物积分\",\"fee\":\"0\",\"coin_id\":\"4\"}]"
				}
			}
		}
		if isFreeze == "1" {
			var tmp = &model.VirtualCoin{
				Id: 4, Name: "购物积分", IsUse: 1, ExchangeRatio: "1",
			}
			m = append(m, tmp)
		}

	}

	return m, nil
}

func VirtualCoinMapInUse(Db *xorm.Engine, masterId, isFreeze string) (map[string]model.VirtualCoin, error) {
	virtualCoinMap := make(map[string]model.VirtualCoin)
	listInUse, err := VirtualCoinListInUse(Db, masterId, isFreeze)
	if err != nil {
		return nil, err
	}
	for _, coin := range listInUse {
		virtualCoinMap[utils.AnyToString(coin.Id)] = *coin
	}
	return virtualCoinMap, nil
}

func VirtualCoinByIds(eg *xorm.Engine, ids []string) map[string]model.VirtualCoin {
	var data []model.VirtualCoin
	virtualCoinMap := make(map[string]model.VirtualCoin)
	err := eg.In("id", ids).Find(&data)
	if err != nil {
		return nil
	}
	for _, coin := range data {
		virtualCoinMap[utils.AnyToString(coin.Id)] = coin
	}
	return virtualCoinMap
}
