package svc

import (
	"applet/app/db"
	"applet/app/db/model"
	model2 "applet/app/flexible_employment/db/model"
	"applet/app/svc"
	"applet/app/utils"
	"errors"
	"time"
	"xorm.io/xorm"
)

func DealFailResult(sess *xorm.Session, apply *model.FinWithdrawApply, masterId, errMsg string) (err error) {
	userProfile, err := db.UserFindByIDWithSession(sess, apply.Uid)
	if err != nil {
		return err
	}

	//1、修改提现单为失败
	apply.State = 3
	apply.Memo = errMsg
	updateAck, err := sess.Where("id=?", apply.Id).Cols("state", "memo").Update(apply)
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		return errors.New("更新提现单失败")
	}
	updateAck, err = sess.Where("request_id=?", apply.Id).Cols("state", "callback_data_for_trade_result").Update(model2.FlexibleEmploymentOrd{
		State:                      3,
		CallbackDataForTradeResult: errMsg,
	})
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		return errors.New("更新工猫记录失败")
	}

	//2、判断类型 加回手续费
	var finUserFlow model.FinUserFlow
	has, err := sess.Where("other_id = ?", apply.Id).Get(&finUserFlow)
	if err != nil {
		return
	}
	var sysFee = 0.00
	if has && apply.FeeType == 1 {
		apply.Amount = utils.Float64ToStr(utils.StrToFloat64(apply.Amount) + utils.StrToFloat64(finUserFlow.SysFee))
		sysFee = utils.StrToFloat64(finUserFlow.SysFee)
	}

	//3、退回余额
	cb, err := svc.HandleBalanceDistributedLock(masterId, utils.IntToStr(apply.Uid), "withdraw_consume")
	if err != nil {
		return
	}
	if cb != nil {
		defer cb()
	}
	var beforeAmount = userProfile.FinValid
	userProfile.FinValid = utils.Float64ToStrPrec4(utils.StrToFloat64(beforeAmount) + utils.StrToFloat64(apply.Amount))
	_, err = db.UserUpdateWithSession(sess, userProfile.Uid, userProfile, "fin_valid")
	if err != nil {
		return err
	}

	//4、插入流水表
	newFinUserFlow := model.FinUserFlow{
		Type:         0,
		Uid:          userProfile.Uid,
		Amount:       apply.Amount,
		BeforeAmount: beforeAmount,
		AfterAmount:  userProfile.FinValid,
		OrdType:      "withdraw",
		OrdAction:    22,
		PaymentType:  1,
		SysFee:       utils.Float64ToStrPrec4(sysFee),
		OrdDetail:    "",
		OtherId:      apply.Id,
		OrdTitle:     "提现退回",
		State:        2,
		OrdTime:      int(apply.CreateAt.Unix()),
		CreateAt:     time.Now(),
		UpdateAt:     time.Now(),
	}
	_, err = db.InsertCommWithSession(sess, &newFinUserFlow)
	if err != nil {
		return err
	}
	return nil
}
func DealFailResultForPuPiao(sess *xorm.Session, apply *model.FinWithdrawApply, masterId, errMsg string) (err error) {
	userProfile, err := db.UserFindByIDWithSession(sess, apply.Uid)
	if err != nil {
		return err
	}

	//1、修改提现单为失败
	apply.State = 3
	apply.Memo = errMsg
	updateAck, err := sess.Where("id=?", apply.Id).Cols("state", "memo").Update(apply)
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		return errors.New("更新提现单失败")
	}
	updateAck, err = sess.Where("request_id=?", apply.Id).Cols("state", "callback_data_for_trade_result").Update(model2.FlexibleEmploymentOrd{
		State:                      3,
		CallbackDataForTradeResult: errMsg,
	})
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		return errors.New("更新工猫记录失败")
	}

	//2、判断类型 加回手续费
	var finUserFlow model.FinUserFlow
	has, err := sess.Where("other_id = ?", apply.Id).Get(&finUserFlow)
	if err != nil {
		return
	}
	var sysFee = 0.00
	if has && apply.FeeType == 1 {
		apply.Amount = utils.Float64ToStr(utils.StrToFloat64(apply.Amount) + utils.StrToFloat64(finUserFlow.SysFee))
		sysFee = utils.StrToFloat64(finUserFlow.SysFee)
	}

	//3、退回余额
	cb, err := svc.HandleBalanceDistributedLock(masterId, utils.IntToStr(apply.Uid), "withdraw_consume")
	if err != nil {
		return
	}
	if cb != nil {
		defer cb()
	}
	var beforeAmount = userProfile.FinValid
	userProfile.FinValid = utils.Float64ToStrPrec4(utils.StrToFloat64(beforeAmount) + utils.StrToFloat64(apply.Amount))
	_, err = db.UserUpdateWithSession(sess, userProfile.Uid, userProfile, "fin_valid")
	if err != nil {
		return err
	}

	//4、插入流水表
	newFinUserFlow := model.FinUserFlow{
		Type:         0,
		Uid:          userProfile.Uid,
		Amount:       apply.Amount,
		BeforeAmount: beforeAmount,
		AfterAmount:  userProfile.FinValid,
		OrdType:      "withdraw",
		OrdAction:    22,
		PaymentType:  1,
		SysFee:       utils.Float64ToStrPrec4(sysFee),
		OrdDetail:    "",
		OtherId:      apply.Id,
		OrdTitle:     "提现退回",
		State:        2,
		OrdTime:      int(apply.CreateAt.Unix()),
		CreateAt:     time.Now(),
		UpdateAt:     time.Now(),
	}
	_, err = db.InsertCommWithSession(sess, &newFinUserFlow)
	if err != nil {
		return err
	}
	return nil
}

func DealSuccessResult(sess *xorm.Session, apply *model.FinWithdrawApply) (err error) {
	//1、修改提现单为成功
	apply.State = 2
	updateAck, err := sess.Where("id=?", apply.Id).Cols("state").Update(apply)
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		return errors.New("更新提现单失败")
	}
	return nil
}
