package aes

import (
	"bytes"
	"crypto/aes"
	"encoding/base64"
	"fmt"
	"net/url"
)

// 加密
func AesEncryptByECB(key []byte, plaintext string) (string, error) {
	block, err := aes.NewCipher(key)
	if err != nil {
		return "", err
	}

	plainBytes := []byte(plaintext)
	// PKCS5Padding
	pad := block.BlockSize() - len(plainBytes)%block.BlockSize()
	plainBytes = append(plainBytes, bytes.Repeat([]byte{byte(pad)}, pad)...)

	// ECB模式
	encrypted := make([]byte, len(plainBytes))
	for bs, be := 0, block.BlockSize(); bs <= len(plainBytes)-block.BlockSize(); bs, be = bs+block.BlockSize(), be+block.BlockSize() {
		block.Encrypt(encrypted[bs:be], plainBytes[bs:be])
	}

	return url.QueryEscape(base64.StdEncoding.EncodeToString(encrypted)), nil
}

// AesDecryptByECB 解密AES ECB加密的字符串
func AesDecryptByECB(key []byte, ciphertext string) (string, error) {

	// 先进行URL解码

	decodedCiphertext, err := url.QueryUnescape(ciphertext)

	if err != nil {

		return "", err

	}

	// Base64解码

	encrypted, err := base64.StdEncoding.DecodeString(decodedCiphertext)

	if err != nil {

		return "", err

	}

	block, err := aes.NewCipher(key)

	if err != nil {

		return "", err

	}

	// ECB模式

	decrypted := make([]byte, len(encrypted))

	for bs, be := 0, block.BlockSize(); bs < len(encrypted); bs, be = bs+block.BlockSize(), be+block.BlockSize() {

		block.Decrypt(decrypted[bs:be], encrypted[bs:be])

	}

	// 去除PKCS5Padding

	decrypted = PKCS5UnPadding(decrypted)

	if decrypted == nil {

		return "", fmt.Errorf("invalid PKCS5 padding")

	}

	return string(decrypted), nil

}

// PKCS5UnPadding 去除PKCS5Padding
func PKCS5UnPadding(data []byte) []byte {
	length := len(data)

	unpadding := int(data[length-1])

	if unpadding > length {

		return nil

	}
	return data[:(length - unpadding)]
}
