package flexible_employment

import (
	"applet/app/cfg"
	"applet/app/utils"
	"bytes"
	"crypto/md5"
	"crypto/tls"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"math/rand"
	"net/http"
	"net/url"
	"sort"
	"strconv"
	"strings"
	"time"
)

type GongMao struct {
	AppKey    string
	AppSecret string
	Nonce     string
	Timestamp string
	ServiceId string
	Sign      string
}

const postUrlForPrd = "https://openapi.gongmall.com"
const postUrlForDev = "https://openapi-qa.gongmall.com"

func New(appKey, appSecret, serviceId string) *GongMao {
	return &GongMao{
		AppKey:    appKey,
		AppSecret: appSecret,
		Nonce:     randomString(32),
		Timestamp: "", // 生成timestamp（当前毫秒时间戳）
		ServiceId: serviceId,
		Sign:      "",
	}
}

// generateSign 生成签名
func (gm *GongMao) generateSign(params map[string]interface{}, appSecret string) string {
	var keys []string
	for k := range params {
		keys = append(keys, k)
	}
	sort.Strings(keys)
	var stringA bytes.Buffer
	for _, k := range keys {
		if stringA.Len() > 0 {
			stringA.WriteString("&")
		}
		stringA.WriteString(url.QueryEscape(k))
		stringA.WriteString("=")
		stringA.WriteString(utils.AnyToString(params[k]))
	}

	stringSignTemp := stringA.String() + "&appSecret=" + appSecret
	h := md5.New()
	h.Write([]byte(stringSignTemp))
	sign := hex.EncodeToString(h.Sum(nil))
	return strings.ToUpper(sign)
}

func (gm *GongMao) Curl(uri string, params map[string]interface{}) (result map[string]interface{}, err error) {
	// 准备请求参数
	gm.Timestamp = strconv.FormatInt(time.Now().UnixNano()/int64(time.Millisecond), 10)
	paramData := make(map[string]interface{})
	paramData = gm.mergeMaps(map[string]interface{}{
		"appKey":    gm.AppKey,
		"nonce":     gm.Nonce,
		"serviceId": gm.ServiceId,
		"timestamp": gm.Timestamp,
	}, params)

	// 生成签名
	paramData["sign"] = gm.generateSign(paramData, gm.AppSecret)

	// 编码请求参数为URL编码的字符串
	form := url.Values{}
	for key, value := range paramData {
		form.Set(key, utils.AnyToString(value))
	}
	requestBody := form.Encode()

	//   构造请求
	var url string
	fmt.Println("prd::::::::::", cfg.Prd)
	if cfg.Prd {
		url = postUrlForPrd + uri
	} else {
		url = postUrlForDev + uri
	}
	fmt.Println("url::::::::::", url)

	req, err := http.NewRequest("POST", url, bytes.NewBufferString(requestBody))
	if err != nil {
		return
	}

	// 设置请求头
	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")

	// 发送请求
	client := &http.Client{
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
		},
	}
	resp, err := client.Do(req)
	if err != nil {
		return
	}
	defer resp.Body.Close()

	// 读取响应体
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return
	}

	// 打印响应内容
	fmt.Printf("Response: %s\n", body)

	// 解析JSON响应
	err = json.Unmarshal(body, &result)
	if err != nil {
		return
	}
	return
}

// 合并map
func (gm *GongMao) mergeMaps(map1, map2 map[string]interface{}) map[string]interface{} {
	for key, value := range map2 {
		map1[key] = value
	}
	return map1
}

// 随机生成指定位数的大写字母和数字的组合
func randomString(ln int) string {
	letters := []rune("1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")
	b := make([]rune, ln)
	r := rand.New(rand.NewSource(time.Now().UnixNano()))
	for i := range b {
		b[i] = letters[r.Intn(len(letters))]
	}

	return string(b)
}
