package flexible_employment

import (
	"bytes"
	"crypto/md5"
	"crypto/tls"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"sort"
	"strconv"
	"time"
)

type PuPiao struct {
	AppId     string
	Charset   string
	Version   string
	Sign      string
	RequestId string
	AppSecret string
}

const urlForPrd = "https://admin.linglong.net.cn"
const urlForDev = "http://api.testlg.cn"

func NewPuPiao(appId, appSecret string) *PuPiao {
	return &PuPiao{
		AppId:     appId,
		Charset:   "UTF-8",
		Version:   "1.0",
		Sign:      "",
		RequestId: "",
		AppSecret: appSecret,
	}
}

// generateSign 生成签名
func (gm *PuPiao) generateSign(params map[string]interface{}, appSecret string) string {
	var keys []string
	for k := range params {
		keys = append(keys, k)
	}
	sort.Strings(keys)
	var stringA = map[string]interface{}{}
	for _, k := range keys {
		stringA[k] = params[k]
	}
	str, _ := json.Marshal(stringA)
	paramJson := string(str)
	stringSignTemp := paramJson + appSecret
	h := md5.New()
	h.Write([]byte(stringSignTemp))
	sign := hex.EncodeToString(h.Sum(nil))
	return sign
}

func (gm *PuPiao) Curl(uri string, params map[string]interface{}) (result map[string]interface{}, err error) {
	// 准备请求参数
	gm.RequestId = strconv.FormatInt(time.Now().UnixNano()/int64(time.Microsecond), 10) //请求Id，每次请求唯一
	paramData := make(map[string]interface{})
	paramData = gm.mergeMaps(map[string]interface{}{
		"appId":     gm.AppId,
		"charset":   gm.Charset,
		"requestId": gm.RequestId,
		"version":   gm.Version,
	}, params)

	// 生成签名
	paramData["sign"] = gm.generateSign(paramData, gm.AppSecret)
	b, err := json.Marshal(paramData)
	if err != nil {
		return
	}

	// 构造请求
	var url string
	if true {
		url = urlForPrd + uri
	} else {
		url = urlForDev + uri
	}
	fmt.Println("url::::::::::", url)
	postBody, err := gm.httpPostBody(url, b)
	if err != nil {
		return
	}
	// 打印响应内容
	fmt.Printf("Response: %s\n", postBody)

	// 解析JSON响应
	err = json.Unmarshal(postBody, &result)
	if err != nil {
		return
	}
	return
}

// 合并map
func (gm *PuPiao) mergeMaps(map1, map2 map[string]interface{}) map[string]interface{} {
	for key, value := range map2 {
		map1[key] = value
	}
	return map1
}

func (gm *PuPiao) httpPostBody(url string, msg []byte) ([]byte, error) {
	client := &http.Client{
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
		},
	}
	resp, err := client.Post(url, "application/json;charset=utf-8", bytes.NewBuffer(msg))
	if err != nil {
		return []byte(""), err
	}
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	return body, err
}
