package consume

import (
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	db2 "code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models/official"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models/official/model"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/streadway/amqp"
	"strings"
	"time"
)

const ZhiOsB2cPlaceOrderNumOfPeopleHashMapCacheKey = "%s:zhiOs_b2c_place_order_num_of_people_hash_map_cache:%s" //下单人数缓存hashMap键

func CanalB2cOrderForNumericalStatementConsume(queue md.MqQueue) {
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(100)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalB2cOrderForNumericalStatementConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleCanalB2cOrderForNumericalStatementTable(res.Body)
			if err != nil {
				fmt.Println("err ::: ", err)
				utils.FilePutContents("CanalB2cOrderForNumericalStatementConsume_ERR", "[err]:"+err.Error())
				_ = res.Reject(false)
				////TODO::重新推回队列末尾，避免造成队列堵塞
				//var msg *md.OneCirclesStructForSignIn
				//json.Unmarshal(res.Body, &msg)
				//ch.Publish(queue.ExchangeName, msg, queue.RoutKey)
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleCanalB2cOrderForNumericalStatementTable(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalB2cOrderMessage[md.CanalB2cOrder]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}
	now := time.Now()

	//2、获取masterId
	masterId := utils.StrToInt(strings.Split(canalMsg.Database, "_")[1])

	//TODO::日志记录
	utils.FilePutContents("handleCanalB2cOrderForNumericalStatementTable_"+utils.IntToStr(masterId)+"_"+now.Format("2006-01-02"), string(msg))

	//2、判断操作(insert | update)
	if canalMsg.Type == md.CanalMsgInsertSqlType || canalMsg.Type == md.CanalMsgUpdateSqlType {
		var isUpdate bool
		//3、查找是否有数据
		var ordDate string
		for _, item := range canalMsg.Data {
			ordDate = utils.TimeParseStd(item.CreateTime).Format("2006-01-02")
		}

		statistics, err := db2.GetMasterB2cOrderStatistics(db.Db, masterId, ordDate)
		if err != nil {
			return err
		}
		if statistics == nil && canalMsg.Type == md.CanalMsgInsertSqlType {
			statistics = &model.MasterB2cOrderStatistics{
				MasterId:              masterId,
				PaymentTotal:          "",
				OrderCount:            0,
				LoseOrderCount:        0,
				PlaceOrderNumOfPeople: 0,
				EffectiveOrderCount:   0,
				EffectiveCommission:   "",
				ReceiveCommission:     "",
				LoseCommission:        "",
				AvgCommission:         "",
				CustomerUnitPrice:     "",
				Date:                  ordDate,
				CreateAt:              now.Format("2006-01-02 15:04:05"),
				UpdateAt:              now.Format("2006-01-02 15:04:05"),
			}
			_, err = db2.MasterB2cOrderStatisticsInsert(db.Db, statistics)
			if err != nil {
				return err
			}
		}
		if statistics == nil {
			return errors.New("过期订单数据不予处理")
		}

		paymentTotal := statistics.PaymentTotal
		orderCount := statistics.OrderCount
		loseOrderCount := statistics.LoseOrderCount
		placeOrderNumOfPeople := statistics.PlaceOrderNumOfPeople
		effectiveOrderCount := statistics.EffectiveOrderCount
		effectiveCommission := statistics.EffectiveCommission
		loseCommission := statistics.LoseCommission
		avgCommission := statistics.AvgCommission
		customerUnitPrice := statistics.CustomerUnitPrice
		effectivePaymentTotal := statistics.EffectivePaymentTotal

		if canalMsg.Type == md.CanalMsgInsertSqlType {
			for _, item := range canalMsg.Data {
				orderCount++
				if item.State == "0" {
					//未支付不统计
					return errors.New("未支付不统计")
				} else if item.State == "6" {
					//失效
					loseOrderCount++
				} else {
					effectiveOrderCount++
				}

				paymentTotal = utils.Float64ToStr(utils.StrToFloat64(paymentTotal) + utils.StrToFloat64(item.CostPrice))                        //付款金额
				effectiveCommission = utils.Float64ToStr(utils.StrToFloat64(effectiveCommission) + utils.StrToFloat64(item.EstimateCommission)) //有效佣金(元)
				effectivePaymentTotal = utils.Float64ToStr(utils.StrToFloat64(effectivePaymentTotal) + utils.StrToFloat64(item.CostPrice))      //有效付款金额(元)

				if item.State != "6" {
					cacheKey := fmt.Sprintf(ZhiOsB2cPlaceOrderNumOfPeopleHashMapCacheKey, utils.IntToStr(masterId), ordDate)
					get, _ := cache.HGetString(cacheKey, item.Uid)
					if get == "" {
						placeOrderNumOfPeople++ //下单人数
						cache.HSet(cacheKey, item.Uid, "1")
					} else {
						cache.HSet(cacheKey, item.Uid, utils.IntToStr(utils.StrToInt(get)+1))
					}
					cache.Expire(cacheKey, md.ZhiOsUserVisitIpAddressHashMapCacheTime)
					paymentTotalValue, _ := decimal.NewFromString(paymentTotal)
					if placeOrderNumOfPeople == 0 {
						return errors.New("divider cannot be 0 in division operation")
					}
					placeOrderNumOfPeopleValue := decimal.NewFromInt(int64(placeOrderNumOfPeople))
					customerUnitPrice = paymentTotalValue.Div(placeOrderNumOfPeopleValue).String()
				}
				//客单价
				isUpdate = true
			}
		}

		if canalMsg.Type == md.CanalMsgUpdateSqlType {
			judgeSate := JudgeB2cOrdSate(*canalMsg)
			if judgeSate > 0 {
				if judgeSate == 2 {
					//TODO::未收货失效
					loseOrderCount++
					effectiveOrderCount--
					for _, item := range canalMsg.Data {
						loseCommission = utils.Float64ToStr(utils.StrToFloat64(loseCommission) + utils.StrToFloat64(item.EstimateCommission))
						effectiveCommission = utils.Float64ToStr(utils.StrToFloat64(effectiveCommission) - utils.StrToFloat64(item.EstimateCommission))
						effectivePaymentTotal = utils.Float64ToStr(utils.StrToFloat64(effectivePaymentTotal) - utils.StrToFloat64(item.CostPrice))
						isUpdate = true
					}
				}
				if judgeSate == 3 {
					//TODO::已收货失效
					loseOrderCount++
					effectiveOrderCount--
					for _, item := range canalMsg.Data {
						loseCommission = utils.Float64ToStr(utils.StrToFloat64(loseCommission) + utils.StrToFloat64(item.EstimateCommission))
						effectiveCommission = utils.Float64ToStr(utils.StrToFloat64(effectiveCommission) - utils.StrToFloat64(item.EstimateCommission))
						effectivePaymentTotal = utils.Float64ToStr(utils.StrToFloat64(effectivePaymentTotal) - utils.StrToFloat64(item.CostPrice))
						isUpdate = true
					}
				}

				if judgeSate == 1 || judgeSate == 3 {
					//TODO::收货额外处理

					//查找是否有数据
					var ordConfirmAt string
					for _, item := range canalMsg.Data {
						ordConfirmAt = utils.TimeParseStd(item.FinishTime).Format("2006-01-02")
					}
					ordConfirmStatistics, err := db2.GetMasterB2cOrderStatistics(db.Db, masterId, ordConfirmAt)
					if err != nil {
						return err
					}
					if ordConfirmStatistics == nil {
						return errors.New("过期收货订单数据不予处理")
					}

					if judgeSate == 1 {
						//TODO::收货
						for _, item := range canalMsg.Data {
							ordConfirmStatistics.ReceiveCommission = utils.Float64ToStr(utils.StrToFloat64(ordConfirmStatistics.ReceiveCommission) + utils.StrToFloat64(item.EstimateCommission))
						}
					}
					if judgeSate == 3 {
						//TODO::已收货失效
						for _, item := range canalMsg.Data {
							ordConfirmStatistics.ReceiveCommission = utils.Float64ToStr(utils.StrToFloat64(ordConfirmStatistics.ReceiveCommission) - utils.StrToFloat64(item.EstimateCommission))
						}
					}
					ordConfirmStatistics.UpdateAt = now.Format("2006-01-02 15:04:05")
					_, err = db2.MasterB2cOrderStatisticsUpdate(db.Db, ordConfirmStatistics.Id, ordConfirmStatistics,
						"receive_commission", "update_at")
				}
			}
		}

		if isUpdate {
			statistics.PaymentTotal = paymentTotal
			statistics.OrderCount = orderCount
			statistics.LoseOrderCount = loseOrderCount
			statistics.PlaceOrderNumOfPeople = placeOrderNumOfPeople
			statistics.EffectiveOrderCount = effectiveOrderCount
			statistics.EffectiveCommission = effectiveCommission
			statistics.LoseCommission = loseCommission
			statistics.AvgCommission = avgCommission
			statistics.CustomerUnitPrice = customerUnitPrice
			statistics.EffectivePaymentTotal = effectivePaymentTotal
			statistics.UpdateAt = now.Format("2006-01-02 15:04:05")
			_, err = db2.MasterB2cOrderStatisticsUpdate(db.Db, statistics.Id, statistics,
				"payment_total", "order_count", "estimated_profit", "lose_order_count",
				"place_order_num_of_people", "effective_order_count", "effective_commission", "lose_commission",
				"avg_commission", "customer_unit_price", "effective_payment_total", "update_at")
			if err != nil {
				return err
			}
		}

	}
	return nil
}

// JudgeB2cOrdSate 处理订单状态(judgeSate[0:不需要处理 1:收货 2:未收货失效 3:已收货失效])
func JudgeB2cOrdSate(message md.CanalB2cOrderMessage[md.CanalB2cOrder]) (judgeSate int) {
	oldData := message.Old

	//1、获取 旧的订单状态
	var oldOrdState string
	for _, item := range oldData {
		if item.State != "" {
			oldOrdState = item.State
		}
	}
	if oldOrdState == "" {
		return
	}

	//2、获取 新的订单状态
	var nowOrdState string
	for _, item := range message.Data {
		if item.State != "" {
			nowOrdState = item.State
		}
	}
	if nowOrdState == "" {
		return
	}

	if oldOrdState == nowOrdState {
		return
	}

	//3、进行状态比较判断
	if oldOrdState == "3" || oldOrdState == "4" || oldOrdState == "5" {
		if nowOrdState == "6" {
			//已收货失效
			return 3
		}
	} else {
		if nowOrdState == "6" {
			//未收货失效
			return 2
		}
		if nowOrdState == "3" || nowOrdState == "4" || nowOrdState == "5" {
			//收货
			return 1
		}
	}
	return
}
