package consume

import (
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	db2 "code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models/official"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models/official/model"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/streadway/amqp"
	"strings"
	"time"
)

const ZhiOsO2oPayPlaceOrderNumOfPeopleHashMapCacheKey = "%s:zhiOs_o2o_pay_place_order_num_of_people_hash_map_cache:%s" //下单人数缓存hashMap键

func CanalO2oPayOrderForNumericalStatementConsume(queue md.MqQueue) {
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(100)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalO2oPayOrderForNumericalStatementConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleCanalO2oPayOrderForNumericalStatementTable(res.Body)
			if err != nil {
				fmt.Println("err ::: ", err)
				utils.FilePutContents("CanalO2oPayOrderForNumericalStatementConsume_ERR", "[err]:"+err.Error())
				_ = res.Reject(false)
				////TODO::重新推回队列末尾，避免造成队列堵塞
				//var msg *md.OneCirclesStructForSignIn
				//json.Unmarshal(res.Body, &msg)
				//ch.Publish(queue.ExchangeName, msg, queue.RoutKey)
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleCanalO2oPayOrderForNumericalStatementTable(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalMallOrder
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return err
	}
	now := time.Now()

	//2、获取masterId
	masterId := utils.StrToInt(strings.Split(canalMsg.Database, "_")[1])

	//TODO::日志记录
	utils.FilePutContents("handleCanalO2oPayOrderForNumericalStatementTable_"+utils.IntToStr(masterId)+"_"+now.Format("2006-01-02"), string(msg))

	//2、判断操作(insert | update)
	if canalMsg.Type == md.CanalMsgInsertSqlType || canalMsg.Type == md.CanalMsgUpdateSqlType {
		var isUpdate bool
		//3、查找是否有数据
		var ordDate string
		for _, item := range canalMsg.Data {
			ordDate = utils.TimeParseStd(item.CreateTime).Format("2006-01-02")
		}

		statistics, err := db2.GetMasterO2oPayOrderStatistics(db.Db, masterId, ordDate)
		if err != nil {
			return err
		}
		if statistics == nil && canalMsg.Type == md.CanalMsgInsertSqlType {
			statistics = &model.MasterO2oPayOrderStatistics{
				MasterId:              masterId,
				PaymentTotal:          "",
				OrderCount:            0,
				EstimatedCommission:   "",
				PlaceOrderNumOfPeople: 0,
				AvgCommission:         "",
				CustomerUnitPrice:     "",
				Date:                  ordDate,
				CreateAt:              now.Format("2006-01-02 15:04:05"),
				UpdateAt:              now.Format("2006-01-02 15:04:05"),
			}
			_, err = db2.MasterO2oPayOrderStatisticsInsert(db.Db, statistics)
			if err != nil {
				return err
			}
		}
		if statistics == nil {
			return errors.New("过期订单数据不予处理")
		}

		paymentTotal := statistics.PaymentTotal
		orderCount := statistics.OrderCount
		estimatedCommission := statistics.EstimatedCommission
		placeOrderNumOfPeople := statistics.PlaceOrderNumOfPeople
		avgCommission := statistics.AvgCommission
		customerUnitPrice := statistics.CustomerUnitPrice

		if canalMsg.Type == md.CanalMsgInsertSqlType {
			for _, item := range canalMsg.Data {
				orderCount++
				if item.State == "0" {
					//未支付不统计
					return errors.New("未支付不统计")
				}
				if item.State == "1" || item.State == "4" {
					paymentTotal = utils.Float64ToStr(utils.StrToFloat64(paymentTotal) + utils.StrToFloat64(item.CostPrice))                        //付款金额
					estimatedCommission = utils.Float64ToStr(utils.StrToFloat64(estimatedCommission) + utils.StrToFloat64(item.EstimateCommission)) //预估佣金(元)

					estimatedCommissionValue, _ := decimal.NewFromString(estimatedCommission)
					orderCountValue := decimal.NewFromInt(int64(orderCount))
					avgCommission = estimatedCommissionValue.Div(orderCountValue).String() //平均佣金

					cacheKey := fmt.Sprintf(ZhiOsO2oPayPlaceOrderNumOfPeopleHashMapCacheKey, utils.IntToStr(masterId), ordDate)
					get, _ := cache.HGetString(cacheKey, item.Uid)
					if get == "" {
						placeOrderNumOfPeople++ //下单人数
						cache.HSet(cacheKey, item.Uid, "1")
					} else {
						cache.HSet(cacheKey, item.Uid, utils.IntToStr(utils.StrToInt(get)+1))
					}
					cache.Expire(cacheKey, md.ZhiOsUserVisitIpAddressHashMapCacheTime)
					paymentTotalValue, _ := decimal.NewFromString(paymentTotal)
					if placeOrderNumOfPeople == 0 {
						return errors.New("divider cannot be 0 in division operation")
					}
					placeOrderNumOfPeopleValue := decimal.NewFromInt(int64(placeOrderNumOfPeople))
					customerUnitPrice = paymentTotalValue.Div(placeOrderNumOfPeopleValue).String() //客单价
					isUpdate = true
				}
			}
		}

		if isUpdate {
			statistics.PaymentTotal = paymentTotal
			statistics.OrderCount = orderCount
			statistics.EstimatedCommission = estimatedCommission
			statistics.PlaceOrderNumOfPeople = placeOrderNumOfPeople
			statistics.AvgCommission = avgCommission
			statistics.CustomerUnitPrice = customerUnitPrice
			statistics.UpdateAt = now.Format("2006-01-02 15:04:05")
			_, err = db2.MasterO2oPayOrderStatisticsUpdate(db.Db, statistics.Id, statistics,
				"payment_total", "order_count", "estimated_commission",
				"place_order_num_of_people",
				"avg_commission", "customer_unit_price", "update_at")
			if err != nil {
				return err
			}
		}

	}
	return nil
}
