package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"applet/mall/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	db2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/svc"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"time"
)

func OneCirclesActivityCoinAutoExchangeGreenEnergyConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>OneCirclesActivityCoinAutoExchangeGreenEnergyConsume>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(50)
	delivery := ch.Consume(queue.Name, false)

	one_circles.Init(cfg.RedisAddr)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			err = handleOneCirclesActivityCoinAutoExchangeGreenEnergyConsume(res.Body)
			if err != nil {
				fmt.Println("OneCirclesActivityCoinAutoExchangeGreenEnergyConsume_ERR:::::", err.Error())
				utils2.FilePutContents("OneCirclesActivityCoinAutoExchangeGreenEnergyConsume_ERR", utils2.SerializeStr(map[string]interface{}{
					"body": res.Body,
					"err":  err.Error(),
				}))
			}
			//_ = res.Reject(false)
			err = res.Ack(true)
			fmt.Println("err ::: ", err)
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleOneCirclesActivityCoinAutoExchangeGreenEnergyConsume(msgData []byte) error {
	time.Sleep(time.Duration(10) * time.Microsecond) //休眠0.1毫秒
	//1、解析mq中queue的数据结构体
	var msg *md2.OneCirclesStructForAutoExchangeGreenEnergy
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}
	engine := db.DBs[msg.MasterId]
	//2、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, cb, err := svc.GetPrice(engine, msg.MasterId)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = logx.Error(err)
		}
	}()
	session.Begin()

	//3给相应的用户减去个人活跃积分
	err = rule.DealUserCoin(session, md2.DealUserCoinReq{
		Kind:         "sub",
		Mid:          msg.MasterId,
		Title:        md2.OneCirclesPersonalActiveCoinExchangeToBeGreenEnergy,
		TransferType: md2.OneCirclesPersonalActiveCoinExchangeToBeGreenEnergyForUserVirtualCoinFlow,
		OrdId:        "",
		CoinId:       msg.CoinId,
		Uid:          msg.Uid,
		Amount:       utils.StrToFloat64(msg.Amount),
	})
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err:::::33333", err.Error())
		return err
	}

	//4.1计算涨价公式
	err1, values, _, afterPriceValue := one_circles.NewCalcPriceIncreaseFormula(msg.AutoExchangeNumsAmount, oneCirclesGreenEnergyBasicSetting)
	if err1 != nil {
		_ = session.Rollback()
		return err1
	}
	//4.2给相应的用户加上个人的绿色积分(可用数量)
	err = rule.DealUserCoin(session, md2.DealUserCoinReq{
		Kind:         "add",
		Mid:          msg.MasterId,
		Title:        md2.OneCirclesPersonalActiveCoinExchangeGreenEnergy,
		TransferType: md2.OneCirclesPersonalActiveCoinExchangeGreenEnergyForUserVirtualCoinFlow,
		OrdId:        "",
		CoinId:       oneCirclesGreenEnergyBasicSetting.PersonGreenEnergyCoinId,
		Uid:          msg.Uid,
		Amount:       utils.StrToFloat64(values),
	})
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err:::::44444", err.Error())
		return err
	}

	//5减少“原始数量”中的绿色能量
	err = one_circles.DealAvailableGreenEnergyCoin(session, int(enum.PersonalActivePointRedemption), utils.StrToFloat64(values), utils.StrToFloat64(msg.AutoExchangeNumsAmount), enum.PersonalActivePointRedemption.String(), oneCirclesGreenEnergyBasicSetting, afterPriceValue)
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err:::::555555", err.Error())
		return err
	}

	//6、修改 one_circles_green_energy_basic_setting 的 now_price
	db2.OneCirclesGreenEnergyBasicSettingUpdate(session, oneCirclesGreenEnergyBasicSetting.Id, oneCirclesGreenEnergyBasicSetting)
	//if err != nil {
	//	_ = session.Rollback()
	//	fmt.Println("err:::::666666", err.Error())
	//	return err
	//}
	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}
	return nil
}
