package db

import (
	"applet/app/utils/logx"
	model2 "applet/super_cloud_issuance/db/model"
	"xorm.io/xorm"
)

// 返回所有, 不管是否显示
func SysModFindAll(Db *xorm.Engine) (*[]model2.SysModule, error) {
	var m []model2.SysModule
	if err := Db.Find(&m); err != nil {
		return nil, err
	}
	return &m, nil
}

// 查找主模块数据
func SysModFindMain(Db *xorm.Engine) (*[]model2.SysModule, error) {
	var m []model2.SysModule
	if err := Db.Where("mod_pid = 0 AND state = 1 AND position = 'base'").
		Asc("sort").
		Find(&m); err != nil {
		return nil, err
	}
	return &m, nil
}

// 用IDS找对应模块数据
func SysModFindByIds(Db *xorm.Engine, ids ...int) (*[]model2.SysModule, error) {
	var m []model2.SysModule
	if err := Db.In("mod_id", ids).Where("state = 1").
		Cols("mod_id,mod_pid,mod_name,position,skip_identifier,title,subtitle,url,margin,aspect_ratio,icon,img,font_color,bg_img,bg_color,bg_color_t,badge,path,data,sort").
		Asc("sort").Find(&m); err != nil {
		return nil, err
	}

	return &m, nil
}

// SysModFindByPosition is 根据位置查找对应模块
func SysModFindByPosition(Db *xorm.Engine, positions ...string) (*[]model2.SysModule, error) {
	var m []model2.SysModule
	if err := Db.In("position", positions).Where("state = 1").Find(&m); err != nil {
		return nil, err
	}
	return &m, nil
}

// SysModFindByTemplateIDAndSkip is 根据模板id 查找对应模块
func SysModFindByTemplateIDAndSkip(Db *xorm.Engine, id interface{}, skip string) (*model2.SysModule, error) {
	var m model2.SysModule
	if has, err := Db.Where("state = 1 AND template_id = ? AND skip_identifier = ?", id, skip).
		Get(&m); err != nil || has == false {
		return nil, logx.Warn(err)
	}
	return &m, nil
}

// SysModFindByTemplateIDAndPID is 根据模板id 和pid =0 查找父模块
func SysModFindByTemplateIDAndPID(Db *xorm.Engine, id interface{}, pid interface{}) (*model2.SysModule, error) {
	var m model2.SysModule
	if has, err := Db.Where("state = 1 AND template_id = ? AND mod_pid = ?", id, pid).
		Get(&m); err != nil || has == false {
		return nil, logx.Warn(err)
	}

	return &m, nil
}

// SysModFindByTemplateIDAndModName is 根据模板id 和mod name 查找模块
func SysModFindByTemplateIDAndModName(Db *xorm.Engine, id interface{}, modName string) (*model2.SysModule, error) {
	var m model2.SysModule
	if has, err := Db.Where("state = 1 AND template_id = ? AND mod_name = ?", id, modName).
		Get(&m); err != nil || has == false {
		return nil, logx.Warn(err)
	}
	return &m, nil
}
