package model

import (
	"time"
)

type WxAppletList struct {
	Id                     int       `json:"id" xorm:"not null pk autoincr INT(10)"`
	Memo                   string    `json:"memo" xorm:"not null default '' comment('备注') unique VARCHAR(255)"`
	CompanyName            string    `json:"company_name" xorm:"not null default '' comment('企业名（需与工商部门登记信息一致）；如果是“无主体名称个体工商户”则填“个体户+法人姓名”，例如“个体户张三”') VARCHAR(255)"`
	Code                   string    `json:"code" xorm:"not null default '' comment('企业代码') VARCHAR(255)"`
	CodeType               int       `json:"code_type" xorm:"not null default 3 comment('企业代码类型 1：统一社会信用代码（18 位） 2：组织机构代码（9 位 xxxxxxxx-x） 3：营业执照注册号(15 位)') TINYINT(3)"`
	LegalPersonaWechat     string    `json:"legal_persona_wechat" xorm:"not null default '' comment('法人微信号') VARCHAR(255)"`
	LegalPersonaName       string    `json:"legal_persona_name" xorm:"not null default '' comment('法人姓名（绑定银行卡）') VARCHAR(255)"`
	State                  int       `json:"state" xorm:"not null default 0 comment('创建状态(0:创建中 1：创建成功 2：创建失败)') TINYINT(3)"`
	Ext                    string    `json:"ext" xorm:"comment('拓展字段') TEXT"`
	UniqueIdentifier       string    `json:"unique_identifier" xorm:"not null default '' comment('唯一标识符(企业名称、企业代码、法人微信、法人姓名四个字段作为每次任务的唯一标示，来区别每次任务。)') VARCHAR(255)"`
	AppId                  string    `json:"app_id" xorm:"default '' comment('小程序appId') VARCHAR(255)"`
	OriginalAppId          string    `json:"original_app_id" xorm:"default '' comment('原始ID') VARCHAR(255)"`
	AuthorizerToken        string    `json:"authorizer_token" xorm:"default '' comment('授权token') VARCHAR(255)"`
	AuthorizerRefreshToken string    `json:"authorizer_refresh_token" xorm:"default '' comment('授权更新token') VARCHAR(255)"`
	AuditVersionState      int       `json:"audit_version_state" xorm:"not null default 0 comment('线上版本号') TINYINT(3)"`
	PublishVersionNum      string    `json:"publish_version_num" xorm:"comment('审核状态(0:暂无审核;1:审核中;2:审核通过;3:审核驳回)') VARCHAR(255)"`
	AppletName             string    `json:"applet_name" xorm:"default '' comment('小程序名字') VARCHAR(255)"`
	AppletSignature        string    `json:"applet_signature" xorm:"default '' comment('小程序简介') VARCHAR(255)"`
	AppletLogo             string    `json:"applet_logo" xorm:"default '' comment('小程序logo') VARCHAR(255)"`
	SetAppletNameInfo      string    `json:"set_applet_name_info" xorm:"default '' comment('小程序改名证件url') VARCHAR(255)"`
	SetAppletNameInfoType  int       `json:"set_applet_name_info_type" xorm:"default 1 comment('小程序改名证件类型(1:个人号 2:组织号)') TINYINT(3)"`
	SetAppletNameState     int       `json:"set_applet_name_state" xorm:"default 0 comment('小程序改名状态(0:未进行 1:进行中 2:改名成功 3:改名失败)') TINYINT(3)"`
	SetAppletNameAuditId   string    `json:"set_applet_name_audit_id" xorm:"default '' comment('小程序改名的审核单id') VARCHAR(255)"`
	CreateAt               time.Time `json:"create_at" xorm:"not null default 'CURRENT_TIMESTAMP' TIMESTAMP"`
	UpdateAt               time.Time `json:"update_at" xorm:"not null default 'CURRENT_TIMESTAMP' TIMESTAMP"`
	IsFilterTaobao         int       `json:"is_filter_taobao" xorm:"default 0 comment('是否过滤淘宝商品 0否 1是') INT(1)"`
	CateId                 string    `json:"cate_id" xorm:"default '' comment('主营类目id') VARCHAR(50)"`
	BottomNavCssId         int       `json:"bottom_nav_css_id" xorm:"default 0 comment('底部导航样式id') INT(11)"`
	AuthType               string    `json:"auth_type" xorm:"default 'reg' comment('授权方式 直接授权old_auth 注册授权reg') VARCHAR(100)"`
	MpAuditVersion         string    `json:"mp_audit_version" xorm:"default '' comment('审核版本') VARCHAR(100)"`
	MpAuditCssId           int       `json:"mp_audit_css_id" xorm:"default 0 comment('审核版本底部样式ID') INT(11)"`
	AppletBgColor          string    `json:"applet_bg_color" xorm:"default '' comment('导航栏/状态栏背景色') VARCHAR(100)"`
	AppletNavColor         string    `json:"applet_nav_color" xorm:"default '' comment('导航栏/状态栏字体色') VARCHAR(100)"`
}
