package db

import (
	"applet/app/db"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/super_cloud_issuance/db/official/model"
	"errors"
	"fmt"
	"reflect"
)

// BatchSelectSuperCloudIssuanceMaster 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `SuperCloudIssuanceMasterFindByParams` 方法
func BatchSelectSuperCloudIssuanceMaster(params map[string]interface{}) (*[]model.SuperCloudIssuanceMaster, error) {
	var SuperCloudIssuanceMasterData []model.SuperCloudIssuanceMaster
	if err := db.Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&SuperCloudIssuanceMasterData); err != nil {
		return nil, logx.Warn(err)
	}
	return &SuperCloudIssuanceMasterData, nil
}

// SuperCloudIssuanceMasterInsert 插入单条数据
func SuperCloudIssuanceMasterInsert(SuperCloudIssuanceMaster *model.SuperCloudIssuanceMaster) (int, error) {
	_, err := db.Db.InsertOne(SuperCloudIssuanceMaster)
	if err != nil {
		return 0, err
	}
	return SuperCloudIssuanceMaster.Id, nil
}

// BatchAddSuperCloudIssuanceMaster 批量新增数据
func BatchAddSuperCloudIssuanceMaster(SuperCloudIssuanceMasterData []*model.SuperCloudIssuanceMaster) (int64, error) {
	affected, err := db.Db.Insert(SuperCloudIssuanceMasterData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetSuperCloudIssuanceMasterCount() int {
	var SuperCloudIssuanceMaster model.SuperCloudIssuanceMaster
	session := db.Db.Where("")
	count, err := session.Count(&SuperCloudIssuanceMaster)
	if err != nil {
		return 0
	}
	return int(count)
}

// SuperCloudIssuanceMasterDelete 删除记录
func SuperCloudIssuanceMasterDelete(id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return db.Db.In("id", id).Delete(model.SuperCloudIssuanceMaster{})
	} else {
		return db.Db.Where("id = ?", id).Delete(model.SuperCloudIssuanceMaster{})
	}
}

// SuperCloudIssuanceMasterUpdate 更新记录
func SuperCloudIssuanceMasterUpdate(id interface{}, SuperCloudIssuanceMaster *model.SuperCloudIssuanceMaster, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = db.Db.Where("id=?", id).Cols(forceColums...).Update(SuperCloudIssuanceMaster)
	} else {
		affected, err = db.Db.Where("id=?", id).Update(SuperCloudIssuanceMaster)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// SuperCloudIssuanceMasterGetOneByParams 通过传入的参数查询数据(单条)
func SuperCloudIssuanceMasterGetOneByParams(params map[string]interface{}) (*model.SuperCloudIssuanceMaster, error) {
	var m model.SuperCloudIssuanceMaster
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := db.Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// SuperCloudIssuanceMasterFindByParams 通过传入的参数查询数据(多条)
func SuperCloudIssuanceMasterFindByParams(params map[string]interface{}) (*[]model.SuperCloudIssuanceMaster, error) {
	var m []model.SuperCloudIssuanceMaster
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := db.Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := db.Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := db.Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}
