package utils

import (
	"errors"
	"time"

	"applet/app/lib/auth"

	"github.com/dgrijalva/jwt-go"
)

// GenToken 生成JWT
func GenToken(uid int, username, phone, appname, MiniOpenID, MiniSK string) (string, error) {
	// 创建一个我们自己的声明
	c := auth.JWTUser{
		uid,
		username,
		phone,
		appname,
		MiniOpenID,
		MiniSK,
		jwt.StandardClaims{
			ExpiresAt: time.Now().Add(auth.TokenExpireDuration).Unix(), // 过期时间
			Issuer:    "zyos",                                          // 签发人
		},
	}
	// 使用指定的签名方法创建签名对象
	token := jwt.NewWithClaims(jwt.SigningMethodHS256, c)
	// 使用指定的secret签名并获得完整的编码后的字符串token
	return token.SignedString(auth.Secret)
}

// ParseToken 解析JWT
func ParseToken(tokenString string) (*auth.JWTUser, error) {
	// 解析token
	token, err := jwt.ParseWithClaims(tokenString, &auth.JWTUser{}, func(token *jwt.Token) (i interface{}, err error) {
		return auth.Secret, nil
	})
	if err != nil {
		return nil, err
	}
	if claims, ok := token.Claims.(*auth.JWTUser); ok && token.Valid { // 校验token
		return claims, nil
	}
	return nil, errors.New("invalid token")
}
